#include <sdcc-lib.h>
#include <pic12f675.h>	// /usr/share/sdcc/non-free/include/pic14/pic12f675.h, /usr/share/sdcc/include/pic14/pic12f675.h

#include "inline.h"
#include "libwait.h"
#include "libusart.h"

__code unsigned char __at 0x2000 id_location[] = {'A', 'B', 'C', 'D'};

unsigned int __at 0x2007 __config =
	_CPD_OFF &
	_CP_OFF &
	_BODEN_ON &	// for 2.1V
	_MCLRE_OFF &
	_PWRTE_OFF &
	_WDT_OFF &
	_INTRC_OSC_NOCLKOUT;

__code unsigned char __at 0x2100 eedata[] = {'d', 'u', 'm', 'm', 'y', 0 };

#define	LED1	GPIO0
#define	DHT11	GPIO1

void Intr(void) __interrupt 0 {
	T0IF = 0;						// Clear timer interrupt
}

/*			  _______
	Vcc		-|	 U	 |-	GND
	NC		-|5		0|-	LED1
	TX		-|4		1|-	DHT11
	NC		-|3_____2|-	MCP9700

			  ___
	DCD(I) >-|1	 \___
			 |		6|-< (I)DSR
	RxD(I) >-|2		 |
			 |		7|-> (O)RTS
	TxD(O) <-|3		 |
			 |		8|-< (I)CTS
	DTR(O) <-|4		 |
			 |	  __9|-< (I)RI
	GND		-|5__/

	GND(5) - RTS(7) 間からわずかに電力が得られる
	RTS(7) は初期 -6V だが「stty crtscts < /dev/ttyUSBx」を実行することで +6V になる
	実際に利用するときは CTS(8) とショートし、常に送信可を示す
*/

unsigned char dht11data[8];
void read_dht11();
unsigned char read_dht11_1();

void main(void) {

	unsigned char i;

				//  --543210
	GPIO		= 0b00000000;		// GPIO3: input only
	CMCON		= 0b00000111;
	ANSEL		= 0b01010100;		// Fosc/16 / GPIO2: Analog
	TRISIO		= 0b00001110;		// 1: 入力, 0: 出力

	OPTION_REG	= 0b11111111;		// 初期値:0b11111111
									// 7:GPIOプルアップ(0:使用する)
									// 6:GP2割り込みポート変化選択(0:立下り)
									// 5:TMR0クロックソース選択ビット(0:内部)
									// 4:TMR0加算タイミング(0:立上り)
									// 3:プリスケーラ割り当てビット(0:TMR0)
									// 2~0プリスケーラレート選択ビット(011:1/16)
									// 4MHz => 1MHz = 1,000,000 / 16 / 256 = 244Hz

	INTCON		= 0b00000000;		// 初期値:0b00000000
									// 7:GIE  Global Interrupt Enable bit
									// 6:PEIE Peripheral Interrupt Enable bit
									// 5:T0IE TMR0 Overflow Interrupt Enable bit
									// 4:INTE GP2/INT External Interrupt Enable bit
									// 3:GPIE GPIO Port Change Interrupt Enable bit
									// 2:T0IF TMR0 Overflow Interrupt Flag bit
									// 1:INTF GP2/INT External Interrupt Flag bit
									// 0:GPIF GPIO Port Change Interrupt Flag bit

	PIR1		= 0b00000000;		// 初期値:0b00000000(フラグ)
	PIE1		= 0b00000000;		// 初期値:0b00000000(イネーブル)
									// 7:EEIE EE Write Complete Interrupt Enable Bit
									// 6:ADIE A/D Converter Interrupt Enable bit
									// 3:CMIE Comparator Interrupt Enable bit
									// 0:TMR1IE TMR1 Overflow Interrupt Enable bit

	T1CON		= 0b00000000;		// 初期値:0b00000000
									// 6:TMR1GE Timer1 Gate Enable bit
									// 5~4:T1CKPS Timer1 Input Clock Prescale Select bits(00:1/1, 11:1/8)
									// 3:T1OSCEN Timer1 Oscillator Enable Control bit(1:Enable)
									// 2:T1SYNC Timer1 External Clock Input Synchronization Control bit(1:Sync)
									// 1:TMR1CS Timer1 Clock Source Select bit(0:Internal)
									// 0:TMR1ON Timer1 On bit(1:Enable)

//	OSCCAL = 0b10111000;			// 2:00.46 4:00.93 6:01.43
	OSCCAL = 0b10111100;			// 1:59.67 3:59.39 5:59.08
	forCalibOsc();

	initUsart();

	for(i = 0; i < 8; i++) {
		dht11data[i] = 0xFF;
	}

	LED1 = 1;
	waitL(10);

	//--------------------------------------------------------------------------
	//
	//	Main Loop
	//
	while(1) {
		LED1 = 0;
		waitL(2);

		ADCON0 = 0b10001001;		// RIGHT / Vdd / AN2 / ADON
		waitS(5);					// 5 x 3 + 6 = 21us 待つ
		GO = 1;
		while(NOT_DONE);
		transmitUsart(ADRESH);
		transmitUsart(ADRESL);

		read_dht11();
		for(i = 0; i < 6; i++) {
			transmitUsart(dht11data[i]);
		}

		LED1 = 1;
		waitL(50);
	}
}

/*------------------------------------------------------------------------------
 *
 *	DHT11 読み出し
 *
 */
void read_dht11() {

	TRISIO &= 0b11111101;			// 0: 出力に変更
	NOP();
	DHT11 = 0;						// L にする

	waitM(40);						// 40 x 500 = 20ms 待つ

	DHT11 = 1;						// H にする
	NOP();
	TRISIO |= 0b00000010;			// 1: 入力に変更

	while(DHT11);					// L になるのを待つ
	while(!DHT11);					// H になるのを待つ

/*	for(j = 0; j < 5; j++) {		// 本当はこう書くところだが、配列への代入処理が遅く、ループ展開することに
		r = 0;
		for(i = 0; i < 8; i++) {
			r <<= 1;
			while(DHT11);			// L になるのを待つ
			while(!DHT11);			// H になるのを待つ
			waitS(10);				// 10 x 3 + 6 = 36us 待つ
			if(DHT11) r += 1;
		}
		dht11data[j] = r;			// これが遅すぎて間に合わない……
	}
*/
	dht11data[0] = read_dht11_1();
	dht11data[1] = read_dht11_1();
	dht11data[2] = read_dht11_1();
	dht11data[3] = read_dht11_1();
	dht11data[4] = read_dht11_1();
}

unsigned char read_dht11_1() {

	unsigned char i, r;

	r = 0;
	for(i = 0; i < 8; i++) {
		r <<= 1;
		while(DHT11);				// L になるのを待つ
		while(!DHT11);				// H になるのを待つ
		waitS(10);					// 10 x 3 + 6 = 36us 待つ
		if(DHT11) r += 1;
	}

	return(r);
}

